#!/usr/bin/perl

# This script reads unicode's NamesList.txt and spits out a block for
# a .mim file.

# Declare which blocks you want to use. You have to give the codepoint
# of the first letter (capital A) and this script then scans all 2*26
# (big and small) letters. Up to now, only the latin and digits
# scripts are supported. Greek will be implemented at a later stage.
@blocks = (
    { start => 0x1D400, script => 'latin', prefix => 'b' },
    { start => 0x1D434, script => 'latin', prefix => 'i' },
    { start => 0x1D49C, script => 'latin', prefix => 's' },
    { start => 0x1D504, script => 'latin', prefix => 'f' },
    { start => 0x1D538, script => 'latin', prefix => 'd' },
    { start => 0x1D7CE, script => 'digits', prefix => 'b' },
    { start => 0x1D7D8, script => 'digits', prefix => 'd' },
);

binmode(STDOUT, ":utf8");

$scripts{'latin'} = 
    [ map { [ $_, ($_<26) ? chr(ord('A')+$_) : chr(ord('a')+$_-26) ] } 0..(26*2-1) ];
$scripts{'digits'} = 
    [ map { [ $_, chr(ord('0')+$_) ] } 0..(9) ];

# Prepare UCD
%UCD = ();
&UCD_read();

# Print header information

# Process blocks
for (@blocks) {
    $_->{'mim_codes'} = [&mim_codes_of_block($_)];
}

# Print latin blocks
my @latin_blocks = grep {$_->{'script'} eq 'latin'} @blocks;
if (@latin_blocks) {
    &print_tabular(@latin_blocks);
}
# Print digits blocks
my @digits_blocks = grep {$_->{'script'} eq 'digits'} @blocks;
if (@digits_blocks) {
    &print_tabular(@digits_blocks);
}
# Print all other blocks
for (grep { $_->{'script'} ne 'latin' and $_->{'script'} ne 'digits' } @blocks) { 
    print $_->{'mim_code'} . "\n" 
}


    


sub mim_codes_of_block {
    my $block = shift;
    my @mim_codes;
    for (@{$scripts{$block->{'script'}}}) {
        $offset = $_->[0]; $access_character = $_->[1];
        # find real codepoint
        my $codepoint = &fix_codepoint(&UCD_int_to_codepoint($block->{'start'} + $offset));
        # the character itself
        my $char = chr(eval "0x$codepoint");
        # create code
        push(@mim_codes, qq[("$block->{'prefix'}$access_character" "$char")]);
    }
    return @mim_codes;
}

sub fix_codepoint {
    # If the codepoint is reserved, the character in question already
    # has a lower codepoint. This lower codepoint is returned.
    # Otherwise the codepoint is returned unchanged.
    my $codepoint = shift;
    unless (&UCD_is_character($codepoint)) {
        return (&UCD_x($codepoint))[0]->[1];
    }
    return $codepoint;
}

sub print_tabular {
    # All blocks @_ are supposed to have the same amount of etnries!
    my @to_print = @_;
    for (my $i = 0; $i < @{$to_print[0]->{'mim_codes'}}; $i++) {
        for (@to_print) {
            print $_->{'mim_codes'}[$i] . " ";
        }
        print "\n";
    }
}


sub UCD_name {
    my $codepoint = shift;
    $UCD{$codepoint} =~ /^[0-9A-F]+\s+(.*)$/m;
    return $1;
}

sub UCD_is_character {
    my $codepoint = shift;
    return &UCD_name($codepoint) =~ /^[^\<]*$/;
}

sub UCD_x {
    # Returns a list of lists
    my $codepoint = shift;
    return map { /\s*x \((.*)\s-\s(.*)\)/ ? [$1,$2] : () } split ("\n", $UCD{$codepoint});
}

sub UCD_int_to_codepoint {
    return sprintf '%04X', shift;
}

sub UCD_read {
    my $NamesListFH;
    open $NamesListFH, '<', 'UNIDATA/NamesList.txt';
    # Returns a hash containing all entries from NamesList.txt using
    # the codepoints as keys.
    $current = undef;
    while (my $line = <$NamesListFH>) {
        if ($line =~ /^\s/) {
            if (defined $current) {
                $UCD{$current} .= $line;
            }
        }
        elsif ($line =~ /^([0-9A-F]+)/) {
            $current = $1;
            $UCD{$current} .= $line;
        }
        else {
            $current = undef;
        }
    }
}

